/* This file is Copyright 2000 Level Control Systems.  See the included LICENSE.txt file for details. */  

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <time.h>

#include "util/NetworkUtilityFunctions.h"
#include "dataio/TCPSocketDataIO.h"
#include "iogateway/MessageIOGateway.h"
#include "reflector/StorageReflectConstants.h"
#include "system/SetupSystem.h"
#include "util/StringTokenizer.h"

using namespace muscle;

#define NET_CLIENT_NEW_CHAT_TEXT 2  // from ShareNetClient.h

// A BeShare-compatible MOTD client for MUSCLE servers.  
// It will connect to the muscled running on the local
// machine, and watch for new BeShare users.  Whenever 
// one arrives, it will send them the motd.txt file.
int main(int argc, char ** argv)
{
   CompleteSetupSystem css;

   while(1)
   {
      int s = Connect("127.0.0.1", 2960, "motdbot", false);
      if (s >= 0)
      {
         MessageIOGateway gw;
         gw.SetDataIO(DataIORef(new TCPSocketDataIO(s, false), NULL));

         // Subscribe to watch all arriving BeShare users
         MessageRef watchBeShareUsers = GetMessageFromPool(PR_COMMAND_SETPARAMETERS);
         watchBeShareUsers()->AddBool("SUBSCRIBE:beshare", true);
         watchBeShareUsers()->AddBool(PR_NAME_SUBSCRIBE_QUIETLY, true);  // so as not to spam when we are startup up
         gw.AddOutgoingMessage(watchBeShareUsers);

         QueueGatewayMessageReceiver inputQueue;
         bool keepGoing = true;
         fd_set readSet, writeSet;
         while(keepGoing)
         {
            FD_ZERO(&readSet);
            FD_ZERO(&writeSet);

            int maxfd = s;
            FD_SET(s, &readSet);

            if (gw.HasBytesToOutput()) FD_SET(s, &writeSet);

            while(keepGoing)
            {
               if (select(maxfd+1, &readSet, &writeSet, NULL, NULL) < 0) 
               {
                  LogTime(MUSCLE_LOG_ERROR, "motdclient: select() failed!\n");
                  keepGoing = false;
               }

               bool reading = FD_ISSET(s, &readSet);
               bool writing = FD_ISSET(s, &writeSet);
               bool writeError = ((writing)&&(gw.DoOutput() < 0));
               bool readError  = ((reading)&&(gw.DoInput(inputQueue) < 0));
               if ((readError)||(writeError))
               {
                  LogTime(MUSCLE_LOG_ERROR, "Connection to server closed!\n");
                  keepGoing = false;
               }

               MessageRef incoming;
               while(inputQueue.RemoveHead(incoming) == B_NO_ERROR)
               {
                  MessageFieldNameIterator it = incoming.GetItemPointer()->GetFieldNameIterator(B_MESSAGE_TYPE);
                  String s;
                  while(it.GetNextFieldName(s) == B_NO_ERROR)
                  {
                     StringTokenizer tok(s.Cstr(), "/");
                     (void) tok.GetNextToken();
                     const char * sessionID = tok.GetNextToken();
                     if (sessionID)
                     {
                        // send him the MOTD!
                        const char * filename = (argc > 1) ? argv[1] : "motd.txt";
                        FILE * f = fopen(filename, "r");
                        if (f)
                        {
                           String motd("\n");
                           char temp[1024];
                           while(fgets(temp, sizeof(temp), f)) motd += temp;

                           MessageRef msg = GetMessageFromPool(NET_CLIENT_NEW_CHAT_TEXT);
                           if (msg())
                           {
                              LogTime(MUSCLE_LOG_INFO, "Sending MOTD message to [%s]\n", sessionID);
                              String target("/*/");
                              target += sessionID;
                              msg()->AddString(PR_NAME_KEYS, target);
                              msg()->AddString("text", motd);
                              msg()->AddString("session", "Message Of The Day");
// uncomment this to make the MOTD appear in blue
//                            msg()->AddBool("private", true);
                              gw.AddOutgoingMessage(msg);
                           }
                           fclose(f);
                        }
                        else LogTime(MUSCLE_LOG_ERROR, "Error, couldn't open [%s]!!!\n", filename);
                     }
                  }
               }

               if ((reading == false)&&(writing == false)) break;

               FD_ZERO(&readSet);
               FD_ZERO(&writeSet);
               FD_SET(s, &readSet);
               if (gw.HasBytesToOutput()) FD_SET(s, &writeSet);
            }
         }
      }
      Snooze64(1000000); // avoid busy-looping
   }
   return 0;
}
